@@ -16,7 +16,7 @@ class Scanner:
 
     def __init__(self, T = Config.T_default, interface = None, scan_clk = None, scan_in = None, scan_out = None,
         scan_en = None, scan_wen = None, fromMSB = False, verbosity = 0, clk_running = False,
-        clk_b = False, length = None, read_func = read_signal):
+        clk_b = False, length = None, read_func = read_signal, scan_in_width = 1):
 
         if interface is None:
             self._scan_in = scan_in
@@ -38,6 +38,7 @@ class Scanner:
         self.clk_b = clk_b
         self.length = length
         self.read_func = read_func
+        self.scan_in_width = scan_in_width
 
         self.halfT = int(T/2)
         self.quarterT = int(T/4)
@@ -75,11 +76,15 @@ class Scanner:
         if read_func is None:
             read_func = self.read_func
 
-        if self.length is not None and len(val) != self.length:
+        if self.length is not None and len(val) != self.length and verbosity > 0:
             print(f'Warning: number of scanned bits ({len(val)}) is not equal to the scan chain length ({self.length})')
 
         if fromMSB:
             val = val[::-1]
+        
+        # Pad and split val to account for >1 scan in width chains
+        val = val.zfill(math.ceil(len(val)/self.scan_in_width))
+        val = [val[i:i+self.scan_in_width] for i in range(0, len(val), self.scan_in_width)]
 
         if Config.running_cocotb:
             if clk_running:
@@ -100,10 +105,10 @@ class Scanner:
         outstr = ''
         for i, v in enumerate(val):
             if scan_out is not None:
-                outstr = outstr + read_func(scan_out)
+                outstr = outstr + read_func(scan_out).zfill(self.scan_in_width)
 
-            if (verbosity == 1 and i%100 == 0) or (verbosity == 2 and i%10 == 0) or (verbosity > 2):
-                print(f"Scanning bit {i} out of {len(val)-1}...")
+            if (verbosity == 3 and i%100 == 0) or (verbosity == 4 and i%10 == 0) or (verbosity > 4):
+                print(f"Scanning bit {i*self.scan_in_width} out of {len(val)*self.scan_in_width-1}...")
 
             write_signal(scan_in, int(v, 2))
 
@@ -113,7 +118,8 @@ class Scanner:
                 cycle(scan_clk, T, clk_running)
 
         if fromMSB:
-            outstr = outstr[::-1]
+            outstr = [outstr[i:i+self.scan_in_width] for i in range(0, len(outstr), self.scan_in_width)]
+            outstr = ''.join(outstr[::-1])
 
         if scan_en is not None:
             write_signal(scan_en, 0)
